@@ -1,5 +1,7 @@
 """Abstract and concrete aggregation types."""
 
+from __future__ import annotations
+
 import abc
 from typing import Callable, ClassVar, Generic, Sequence, Tuple
 
@@ -20,7 +22,7 @@ class Aggregate(Generic[Output], abc.ABC):
         possible_peers: Sequence[Tuple[int, AbstractRow]],
         getters: Tuple[Getter, ...],
         order_by_columns: Sequence[str],
-    ) -> Aggregator["Aggregate[Output]", Output]:
+    ) -> Aggregator[Aggregate[Output], Output]:
         """Prepare an aggregation of this type for computation."""
         arguments = [
             tuple(getter(peer) for getter in getters) for _, peer in possible_peers
