@@ -12,6 +12,8 @@
 
 """
 
+from __future__ import annotations
+
 import inspect
 from typing import Any, Callable, Iterable, Mapping, Optional
 
@@ -50,6 +52,7 @@ from .core import (
     Join,
     JoinPredicate,
     LeftJoin,
+    Limit,
     Mutate,
     PartitionBy,
     Predicate,
@@ -80,7 +83,7 @@ class _shiftable(curry):
     def __signature__(self) -> inspect.Signature:
         return inspect.signature(self.func)  # pragma: no cover
 
-    def __rrshift__(self, other: Relation) -> "_shiftable":
+    def __rrshift__(self, other: Relation) -> _shiftable:
         return self(other)
 
 
@@ -628,6 +631,27 @@ def difference_all(right: Relation, left: Relation) -> DifferenceAll:
     return DifferenceAll(left, right)
 
 
+@_shiftable
+def limit(limit: int, relation: Relation, *, offset: int = 0) -> Limit:
+    """Return the rows in `relation` starting from `offset` up to `limit`.
+
+    Parameters
+    ----------
+    limit
+        The number of rows starting from offset to produce
+    relation
+        Relation whose rows to limit
+    offset
+        The number of rows to skip before yielding
+
+    """
+    if offset < 0:
+        raise ValueError(f"invalid offset, must be non-negative: {offset}")
+    if limit < 0:
+        raise ValueError(f"invalid limit, must be non-negative: {limit}")
+    return Limit(relation, offset=offset, limit=limit)
+
+
 # Aggregate functions
 def count(x: Callable[[AbstractRow], Optional[T]]) -> AggregateSpecification:
     """Count the number of non-NULL values of `x`.
