@@ -12,6 +12,8 @@ features up to rule-based query optimization.
 
 """
 
+from __future__ import annotations
+
 import abc
 import collections
 import functools
@@ -52,40 +54,22 @@ from stupidb.typehints import (
 )
 
 
-class Partitionable(abc.ABC):
-    """An abstraction for a set of rows that can be partitioned by a key."""
+class Relation(abc.ABC):
+    """An abstract relation."""
 
-    __slots__ = ("rows",)
+    __slots__ = "child", "partitioners"
 
-    def __init__(self, rows: Iterable[AbstractRow]) -> None:
-        self.rows = rows
-
-    def partition_key(self, row: AbstractRow) -> PartitionKey:
-        return ()
-
-    def __iter__(self) -> Iterator[AbstractRow]:
-        return iter(self.rows)
-
-
-class Relation(Partitionable):
-    """A relation."""
-
-    __slots__ = ("child",)
-
-    def __init__(self, child: Partitionable) -> None:
+    def __init__(self, child: Iterable[AbstractRow]) -> None:
         self.child = child
+        self.partitioners: Mapping[str, PartitionBy] = {}
 
     def __iter__(self) -> Iterator[AbstractRow]:
         """Iterate over the rows of a :class:`~stupidb.stupidb.Relation`."""
         return (row._renew_id(id) for id, row in enumerate(filter(None, self.child)))
 
     @classmethod
-    def from_iterable(cls, iterable: Iterable[Mapping[str, Any]]) -> "Relation":
-        return cls(
-            Partitionable(
-                Row.from_mapping(row, _id=i) for i, row in enumerate(iterable)
-            )
-        )
+    def from_iterable(cls, iterable: Iterable[Mapping[str, Any]]) -> Relation:
+        return cls(Row.from_mapping(row, _id=i) for i, row in enumerate(iterable))
 
 
 FullProjector = Union_[Projector, WindowAggregateSpecification]
@@ -170,9 +154,7 @@ class Mutate(Projection):
         # we need to use it twice: once for the computed columns (self.child)
         # used during the iteration of super().__iter__() and once for the
         # original relation (child)
-        child, self.child = typing.cast(
-            Tuple[Partitionable, Partitionable], itertools.tee(self.child)
-        )
+        child, self.child = itertools.tee(self.child)
         return (
             Row.from_mapping(row, _id=i)
             for i, row in enumerate(map(toolz.merge, child, super().__iter__()))
@@ -211,20 +193,19 @@ class Aggregation(Generic[AssociativeAggregate], Relation):
                 name: aggspec.aggregate_type() for name, aggspec in aggregations.items()
             }
         )
-        child = self.child
+
+        child = typing.cast(Relation, self.child)
         for row in child:
-            key = child.partition_key(row)
-            aggs: Mapping[str, AssociativeAggregate] = grouped_aggs[key]
-            for name, agg in aggs.items():
-                inputs = [getter(row) for getter in aggregations[name].getters]
+            key = tuple(
+                (name, keyfunc(row)) for name, keyfunc in child.partitioners.items()
+            )
+            for name, agg in grouped_aggs[key].items():
+                inputs = (getter(row) for getter in aggregations[name].getters)
                 agg.step(*inputs)
 
         for id, (grouping_key, aggs) in enumerate(grouped_aggs.items()):
             data = dict(grouping_key)
-            finalized_aggregations = {
-                name: agg.finalize() for name, agg in aggs.items()
-            }
-            data.update(finalized_aggregations)
+            data.update((name, agg.finalize()) for name, agg in aggs.items())
             yield Row(data, _id=id)
 
 
@@ -263,14 +244,11 @@ class GroupBy(Relation):
 
     """
 
-    __slots__ = ("group_by",)
+    __slots__ = "group_by", "partitioners"
 
     def __init__(self, child: Relation, group_by: Mapping[str, PartitionBy]) -> None:
         super().__init__(child)
-        self.group_by = group_by
-
-    def partition_key(self, row: AbstractRow) -> PartitionKey:
-        return tuple((name, keyfunc(row)) for name, keyfunc in self.group_by.items())
+        self.partitioners = group_by
 
 
 class SortBy(Relation):
@@ -296,16 +274,31 @@ class SortBy(Relation):
         self.nulls = nulls
 
     def __iter__(self) -> Iterator[AbstractRow]:
-        return iter(
-            sorted(
-                self.child,
-                key=functools.cmp_to_key(
-                    functools.partial(row_key_compare, self.order_by, self.nulls)
-                ),
+        return (
+            row._renew_id(id)
+            for id, row in enumerate(
+                sorted(
+                    self.child,
+                    key=functools.cmp_to_key(
+                        functools.partial(row_key_compare, self.order_by, self.nulls)
+                    ),
+                )
             )
         )
 
 
+class Limit(Relation):
+    __slots__ = "offset", "limit"
+
+    def __init__(self, child: Relation, *, offset: int, limit: int) -> None:
+        super().__init__(child)
+        self.offset = offset
+        self.limit = limit
+
+    def __iter__(self) -> Iterator[AbstractRow]:
+        return itertools.islice(self.child, self.offset, self.offset + self.limit, None)
+
+
 class Join(Relation):
     __slots__ = "left", "right", "predicate"
 
@@ -316,12 +309,8 @@ class Join(Relation):
         self.right, right_ = itertools.tee(right)
         self.predicate = predicate
         super().__init__(
-            Partitionable(
-                JoinedRow(left_row, right_row, _id=i)
-                for i, (left_row, right_row) in enumerate(
-                    itertools.product(left_, right_)
-                )
-            )
+            JoinedRow(left_row, right_row, _id=i)
+            for i, (left_row, right_row) in enumerate(itertools.product(left_, right_))
         )
 
     def __iter__(self) -> Iterator[AbstractRow]:
@@ -405,17 +394,16 @@ class RightJoin(AsymmetricJoin):
 
     @property
     def matching_relation(self) -> Iterator[AbstractRow]:
+        """Return the relation that should be used for matching."""
         return self.right
 
     def mismatch_keys(self, row: AbstractRow) -> Set[str]:
+        """Return the keys that should be used for mismatch detection."""
         return row.left.keys()
 
 
-SetOperand = FrozenSet[Tuple[Tuple[str, Any], ...]]
-
-
 class SetOperation(Relation):
-    """A generic set operation."""
+    """An abstract set operation."""
 
     __slots__ = "left", "right"
 
@@ -424,11 +412,16 @@ class SetOperation(Relation):
         self.right = right
 
     @staticmethod
-    def itemize(mappings: Iterable[AbstractRow]) -> SetOperand:
+    def itemize(
+        mappings: Iterable[AbstractRow],
+    ) -> FrozenSet[Tuple[Tuple[str, Any], ...]]:
+        """Return a hashable version of `mappings`."""
         return frozenset(tuple(mapping.items()) for mapping in mappings)
 
 
 class Union(SetOperation):
+    """Union between two relations."""
+
     __slots__ = ()
 
     def __iter__(self) -> Iterator[AbstractRow]:
@@ -444,6 +437,8 @@ class Union(SetOperation):
 
 
 class UnionAll(SetOperation):
+    """Non-unique union between two relations."""
+
     __slots__ = ()
 
     def __iter__(self) -> Iterator[AbstractRow]:
@@ -454,6 +449,8 @@ class UnionAll(SetOperation):
 
 
 class IntersectAll(SetOperation):
+    """Non-unique intersection between two relations."""
+
     __slots__ = ()
 
     def __iter__(self) -> Iterator[AbstractRow]:
@@ -478,6 +475,8 @@ class IntersectAll(SetOperation):
 
 
 class Difference(SetOperation):
+    """Unique difference between two relations."""
+
     __slots__ = ()
 
     def __iter__(self) -> Iterator[AbstractRow]:
@@ -493,6 +492,8 @@ class Difference(SetOperation):
 
 
 class DifferenceAll(SetOperation):
+    """Non-unique difference between two relations."""
+
     __slots__ = ()
 
     def __iter__(self) -> Iterator[AbstractRow]:
@@ -506,6 +507,8 @@ class DifferenceAll(SetOperation):
 
 
 class Intersect(SetOperation):
+    """Intersection of two relations."""
+
     __slots__ = ()
 
     def __iter__(self) -> Iterator[AbstractRow]:
